#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <curl/curl.h>

int seek_callback(void *userp, curl_off_t offset, int origin)
{
	FILE* file_handle = (FILE*)userp;
	printf("Calling seek function\n");
	return (fseek(file_handle, offset, origin) == 0) ? CURL_SEEKFUNC_OK : CURL_SEEKFUNC_FAIL;
}

int main(int argc, char *argv[])
{
	struct curl_slist *header_list = NULL;
	struct curl_httppost *post = NULL;
	struct curl_httppost *last = NULL;
	long file_size;
	struct stat file_stat;
	FILE* file_handle;
	CURL* curl_handle;
	int i;

	if (argc < 2)
	{
		printf("Specify a file\n");
		return 1;
	}

	curl_global_init(CURL_GLOBAL_ALL);
	curl_handle = curl_easy_init();

	// Open file & get size
	file_handle = fopen(argv[1], "r+");
	stat(argv[1], &file_stat);
	file_size = file_stat.st_size;

	curl_formadd(&post,
		&last,
		CURLFORM_COPYNAME, "file_upload",
		CURLFORM_STREAM, file_handle,
		CURLFORM_CONTENTSLENGTH, file_size,
		CURLFORM_FILENAME, "upload_file",
		CURLFORM_END);
	curl_formadd(&post,
		&last,
		CURLFORM_COPYNAME, "save_name",
		CURLFORM_COPYCONTENTS, "test_file_post",
		CURLFORM_END);

	header_list = curl_slist_append(header_list, "Expect:");

	curl_easy_setopt(curl_handle, CURLOPT_VERBOSE, 1L);
	curl_easy_setopt(curl_handle, CURLOPT_FAILONERROR, 1L);
	curl_easy_setopt(curl_handle, CURLOPT_NOPROGRESS, 1L);
	curl_easy_setopt(curl_handle, CURLOPT_HEADER, 0L);

	curl_easy_setopt(curl_handle, CURLOPT_URL, "http://127.0.0.1:8080/testpost");
	curl_easy_setopt(curl_handle, CURLOPT_HTTPPOST, post);
	curl_easy_setopt(curl_handle, CURLOPT_HTTPHEADER, header_list);
	curl_easy_setopt(curl_handle, CURLOPT_SEEKFUNCTION, seek_callback);
	curl_easy_setopt(curl_handle, CURLOPT_SEEKDATA, file_handle);

	for (i = 0; i < 10000; i++)
	{
		CURLcode retval = curl_easy_perform(curl_handle);
		if (retval != CURLE_OK)
		{
			printf("Error %d from curl_easy_perform\n", retval);
			if (retval == CURLE_COULDNT_CONNECT)
			{
				printf("Couldn't connect, exiting\n");
				return 1;
			}
		}
		fseek(file_handle, 0, SEEK_SET);
		usleep((rand() % 100));
	}

	// Clean up
	curl_formfree(post);
	curl_slist_free_all(header_list);
	fclose(file_handle);

	return 0;
}
